package zero1hd.rhythmbullet.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputMultiplexer;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.ImageButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.BasicMusicInfo;
import zero1hd.rhythmbullet.audio.map.GamePlayMap;
import zero1hd.rhythmbullet.stages.GamePlayArea;
import zero1hd.rhythmbullet.stages.ui.GameHUD;

public class GameScreen extends ScreenAdapter {
	private GamePlayArea gameArea;
	private GameHUD gameHUD;
	
	private InputMultiplexer inputs;

	public RhythmBullet core;
	
	private BasicMusicInfo music;
	
	SpriteBatch bgBatch;
	private ShaderProgram bgShader;
	private Texture background;
	
	public GameScreen(RhythmBullet core, Screen screen) {
		this.core = core;
		
		// Overlay stuff
		ImageButton pause = new ImageButton(core.getDefaultSkin().getDrawable("pause"),
				core.getDefaultSkin().getDrawable("pause-down"));
		pause.setPosition(Gdx.graphics.getWidth() - pause.getWidth() - 25,
				Gdx.graphics.getHeight() - pause.getHeight() - 25);
		pause.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				pause();
			}
		});
		
		gameArea = new GamePlayArea(core.getAssetManager(), core.getPrefs());
		gameHUD = new GameHUD(core.getDefaultSkin(), 100f, gameArea, screen, core);
		inputs = new InputMultiplexer();
		inputs.addProcessor(gameHUD);
		inputs.addProcessor(gameArea);
		
		background = core.getAssetManager().get("star_bg.png");
		
		if (core.getPrefs().getBoolean("bg shader")) {
			Gdx.app.debug("Shader", "using background shader");
			
			bgShader = new ShaderProgram(Gdx.files.internal("shaders/bg.vsh"), Gdx.files.internal("shaders/bg.fsh"));
			
			if (!bgShader.isCompiled()) {
				System.err.println(bgShader.getLog());
				System.exit(0);
			}
			if (bgShader.getLog().length()!=0) {
				System.out.println(bgShader.getLog());
			}
			
			bgBatch = new SpriteBatch(2, bgShader);
		} else {
			bgBatch = new SpriteBatch(2);
		}
	}
	
	public void setGamePlayMap(GamePlayMap gpm) {
		music = gpm.getMusicData();
		gameArea.setAudioMap(gpm);
		gameHUD.setMusic(gpm.getMusicData().getPlaybackMusic());
	}
	
	@Override
	public void show() {
		if (music == null) new NullPointerException("Idiot, you can't have a music game not have music on the gameplay screen...");
		Gdx.input.setInputProcessor(inputs);
		gameArea.loadShaders(core.getPrefs());
		
		if (!gameHUD.isPaused()) {
			music.getPlaybackMusic().play();
		}
		
		super.show();
	}

	@Override
	public void render(float delta) {
		Gdx.gl.glClearColor(0f, 0f, 0f, 0f);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		//Background stuff should literally span the whole screen so no matrice stuff
		bgBatch.begin();
		if (bgShader != null) {
			bgBatch.setShader(bgShader);
			bgShader.setUniformf("resolution", Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			
			if (music != null) {
				bgShader.setUniformf("time", music.getPlaybackMusic().getPosition());
			}
		}
		bgBatch.draw(background, 0f, 0f);
		bgBatch.end();
		
		
		//actual game and hud
		if (!gameHUD.isPaused()) {
			gameArea.act(delta);
			if (gameArea.getPolyjetEntity().isDead()) {
				end(false);
			}
			gameHUD.act(delta);
		}
		gameArea.getViewport().apply();
		gameArea.draw();
		gameHUD.getViewport().apply();
		gameHUD.draw();
		if (!music.getPlaybackMusic().isPlaying()) {
			end(true);
		}
		
		super.render(delta);
	}

	private void end(boolean trueEnd) {
		if (trueEnd) {
			// TODO they didn't die
		} else {
			// TODO They done goofed and ended up dying.
		}
	}

	@Override
	public void dispose() {
		gameHUD.dispose();
		gameArea.dispose();
		super.dispose();
	}
	
	@Override
	public void pause() {
		gameHUD.setPaused(true);
		super.pause();
	}
	
	@Override
	public void resize(int width, int height) {
		gameArea.getViewport().update(width, height, true);
		gameHUD.getViewport().update(width, height, true);

		super.resize(width, height);
	}
	
	public GamePlayArea getGameArea() {
		return gameArea;
	}
}