package zero1hd.rhythmbullet.stages.ui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.CheckBox;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.Window;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.AudioAnalyzer;
import zero1hd.rhythmbullet.audio.map.RhythmMapAlgorithm;
import zero1hd.rhythmbullet.screens.MainMenu;
import zero1hd.rhythmbullet.stages.GamePlayArea;
import zero1hd.rhythmbullet.ui.windows.BassUMGraphWindow;
import zero1hd.rhythmbullet.ui.windows.BeatViewer;
import zero1hd.rhythmbullet.ui.windows.DifficultyWindow;
import zero1hd.rhythmbullet.ui.windows.FPSWindow;
import zero1hd.rhythmbullet.ui.windows.MGraphWindow;
import zero1hd.rhythmbullet.ui.windows.MusicController;
import zero1hd.rhythmbullet.ui.windows.MusicSelector;
import zero1hd.rhythmbullet.ui.windows.SpawnerWindow;
import zero1hd.rhythmbullet.ui.windows.VolumeWindow;
import zero1hd.rhythmbullet.util.MiniEvents;
import zero1hd.rhythmbullet.util.MiniListener;

public class CreativeHUD extends Stage implements MiniListener {
	MusicController musicPlayBackControls;
	MusicSelector musicSelector;
	FPSWindow fpsViewer;
	BeatViewer beatViewer;
	BassUMGraphWindow bassUMgraphWindow;
	MGraphWindow mGraphWindow;
	VolumeWindow volumeWindow;
	SpawnerWindow spawnerWindow;
	DifficultyWindow diffWindow;
	
	AudioAnalyzer analyzer;
	RhythmMapAlgorithm mapGen;
	Window toolbox;
	
	GamePlayArea gpa;
	GameHUD ghud;
	public CreativeHUD(final RhythmBullet core, final MainMenu mainMenu, final GamePlayArea gpa, GameHUD ghud) {
		this.gpa = gpa;
		this.ghud = ghud;
		
		musicSelector = new MusicSelector("Select Audio File", core.getDefaultSkin(), core.getPrefs().getString("music dir"), "default");
		musicSelector.miniSender.addListener(this);
		musicSelector.refresh();

		fpsViewer = new FPSWindow("FPS", core.getDefaultSkin());
		beatViewer = new BeatViewer("Beat", core.getDefaultSkin());
		bassUMgraphWindow = new BassUMGraphWindow("Bass/UM Peak Values", core.getDefaultSkin());
		mGraphWindow = new MGraphWindow("Midrange Peak Values", core.getDefaultSkin());
		volumeWindow = new VolumeWindow("Volume adjustments", core.getDefaultSkin(), core.getPrefs());
		spawnerWindow = new SpawnerWindow("Spawn Tool", core.getDefaultSkin(), gpa.em, gpa.cm, gpa);
		diffWindow = new DifficultyWindow(core.getDefaultSkin());
		
		toolbox = new Window("Tools", core.getDefaultSkin(), "tinted");
		toolbox.defaults().pad(5f);
		
		Table toolboxToolSet = new Table(core.getDefaultSkin());
		toolboxToolSet.defaults().space(5f).left();
		
		//Back button
		TextButton backButton = new TextButton("Back", core.getDefaultSkin(), "window");
		backButton.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				dispose();
				core.setScreen(mainMenu);
			}
		});
		toolboxToolSet.add(backButton);
		toolboxToolSet.row();

		final CheckBox musicSelectorCheckbox = new CheckBox(" Music selector", core.getDefaultSkin());
		musicSelectorCheckbox.addListener(new ChangeListener() {

			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (musicSelectorCheckbox.isChecked()) {
					addActor(musicSelector);
				} else {
					musicSelector.remove();
				}
			}
		});
		toolboxToolSet.add(musicSelectorCheckbox);
		toolboxToolSet.row();
		
		musicPlayBackControls = new MusicController(core.getDefaultSkin());
		final CheckBox musicPlayBackCheckbox = new CheckBox(" Playback controls", core.getDefaultSkin());
		musicPlayBackCheckbox.addListener(new ChangeListener() {

			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (musicPlayBackCheckbox.isChecked()) {
					addActor(musicPlayBackControls);
				} else {
					musicPlayBackControls.remove();
				}
			}
		});
		toolboxToolSet.add(musicPlayBackCheckbox);
		toolboxToolSet.row();
		
		final CheckBox beatViewerCheckbox = new CheckBox(" Beat viewer", core.getDefaultSkin());
		beatViewerCheckbox.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (beatViewerCheckbox.isChecked()) {
					addActor(beatViewer);
				} else {
					beatViewer.remove();
				}
			}
		});
		toolboxToolSet.add(beatViewerCheckbox);
		toolboxToolSet.row();
		
		final CheckBox fpsViewerCheckbox = new CheckBox(" FPS", core.getDefaultSkin());
		fpsViewerCheckbox.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (fpsViewerCheckbox.isChecked()) {
					addActor(fpsViewer);
				} else {
					fpsViewer.remove();
				}
			}
		});
		toolboxToolSet.add(fpsViewerCheckbox);
		toolboxToolSet.row();
		
		final CheckBox peakGraphCheckbox = new CheckBox(" Peak Graphs", core.getDefaultSkin());
		peakGraphCheckbox.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (peakGraphCheckbox.isChecked()) {
					addActor(bassUMgraphWindow);
					addActor(mGraphWindow);
				} else {
					bassUMgraphWindow.remove();
					mGraphWindow.remove();
				}
				
			}
		});
		toolboxToolSet.add(peakGraphCheckbox);
		toolboxToolSet.row();
		
		final CheckBox volumeMixerCheckbox = new CheckBox(" Volume", core.getDefaultSkin());
		volumeMixerCheckbox.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (volumeMixerCheckbox.isChecked()) {
					addActor(volumeWindow);
				} else {
					volumeWindow.remove();
				}
			}
		});
		toolboxToolSet.add(volumeMixerCheckbox);
		toolboxToolSet.row();
		
		final CheckBox spawnToolCheckBox = new CheckBox(" Spawn Tool", core.getDefaultSkin());
		spawnToolCheckBox.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (spawnToolCheckBox.isChecked()) {
					addActor(spawnerWindow);
				} else {
					spawnerWindow.remove();
				}
			}
		});
		toolboxToolSet.add(spawnToolCheckBox);
		toolboxToolSet.row();
		
		final CheckBox diffAdjusterCheckBox = new CheckBox(" Difficulty", core.getDefaultSkin());
		diffAdjusterCheckBox.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (diffAdjusterCheckBox.isChecked()) {
					addActor(diffWindow);
				} else {
					diffWindow.remove();
				}
			}
		});
		toolboxToolSet.add(diffAdjusterCheckBox);
		
		ScrollPane scroller = new ScrollPane(toolboxToolSet, core.getDefaultSkin());
		toolbox.add(scroller).height(250f).width(150f);
		addActor(toolbox);
		
		addListener(new InputListener() {
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				if (keycode == Keys.ENTER || keycode == Keys.ESCAPE) {
					setKeyboardFocus(null);
				}
				return super.keyUp(event, keycode);
			}
		});
		toolbox.pack();
	}

	@Override
	public void act(float delta) {
		super.act(delta);
	}
	
	@Override
	public void dispose() {
		if (analyzer != null) {
			analyzer.getAudioDataPackage().getMusicInfo().dispose();
		}
		super.dispose();
	}

	@Override
	public void handle(MiniEvents ID) {
		switch (ID) {
		case BACK:
			
			break;
		case MUSIC_SELECTED:
			beatViewer.setMusic(null, null);
			volumeWindow.setMusic(null);
			bassUMgraphWindow.setData(null, null, null);
			mGraphWindow.setData(null, null, null);
			if (musicPlayBackControls.getAudiofile() != null) {
				musicPlayBackControls.getAudiofile().dispose();
			}
			musicPlayBackControls.setAudiofile(null);
			analyzer = new AudioAnalyzer();
			analyzer.sender.addListener(this);

			analyzer.startAnalyticalThread(musicSelector.getSelectedMusic());
			break;
		case SPECTRAL_FLUX_DONE:
			analyzer.runThresholdCleaning();
			break;
		case MUSIC_DATA_CLEANED:
			if (analyzer != null && analyzer.isFinalized()) {
				mapGen = new RhythmMapAlgorithm(gpa.em, gpa.cm, analyzer.getAudioDataPackage(), diffWindow.getSpeedModifier(), diffWindow.getHealthModifier(), diffWindow.getOverallDiffMod());
				mapGen.getSender().addListener(this);
				Thread mapGenThread = new Thread(mapGen);
				mapGenThread.start();
			}
			break;
		case MAP_GENERATED:
			Gdx.app.debug("creative", "successfully generated map.");
			musicPlayBackControls.setAudiofile(analyzer.getAudioData());
			volumeWindow.setMusic(analyzer.getAudioData());
			beatViewer.setMusic(analyzer.getAudioData(), analyzer.getAudioDataPackage());
			ghud.setMusic(null);
			
			bassUMgraphWindow.setData(analyzer.getAudioDataPackage().getBassPeaks(), analyzer.getAudioDataPackage().getuMPeaks(), analyzer.getAudioData());
			bassUMgraphWindow.getGraph().avgG1 = analyzer.getBassAvg();
			bassUMgraphWindow.getGraph().normalDataG1 = analyzer.getBassMaxValue();
			bassUMgraphWindow.getGraph().avgG2 = analyzer.getUmAvg();
			bassUMgraphWindow.getGraph().normalDataG2 = analyzer.getUMMaxValue();
			
			mGraphWindow.setData(analyzer.getAudioDataPackage().getmPeaks(), null, analyzer.getAudioData());
			mGraphWindow.getGraph().normalDataG1 = analyzer.getmMaxValue();
			mGraphWindow.getGraph().avgG1 = analyzer.getmAvg();
			gpa.setAudioMap(mapGen.getMap());
			ghud.setMusic(analyzer.getAudioData().getPlaybackMusic());
			break;
		default:
			break;
		}
	}
}
