package zero1hd.rhythmbullet.ui.pages;

import java.io.File;
import java.io.FilenameFilter;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.AudioInfo;
import zero1hd.rhythmbullet.ui.builders.MusicSelectable;
import zero1hd.rhythmbullet.ui.windows.LoadingWindow;
import zero1hd.rhythmbullet.ui.windows.NoticeWindow;
import zero1hd.rhythmbullet.util.MiniEvents;

public class MusicSelectionPage extends Page {
	private volatile Table musicChoices;
	Preferences musicFileAnnotation;
	private RhythmBullet core;
	private volatile ScrollPane musicChoiceScroller;
	
	private volatile LoadingWindow loadingWindow;
	protected volatile boolean cancel;
	private TextButton back;
	
	private FileHandle selectedMusic;
	private AudioInfo selectedMusicInfo;
	
	public MusicSelectionPage(final RhythmBullet core) {
		super("Select music", core.getDefaultSkin());
		this.core = core;
		
		musicFileAnnotation = Gdx.app.getPreferences("music_file_annotation");
		
		back = new TextButton("Back", core.getDefaultSkin());
		back.setPosition(getWidth()-back.getWidth()-15f, getHeightBelowTitle());
		back.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				miniSender.send(MiniEvents.BACK);
				cancel = true;
			}
		});
		addActor(back);
		
		loadingWindow = new LoadingWindow(core.getDefaultSkin(), "tinted", true, core.getAssetManager(), core.getPrefs().getFloat("fx vol"));
		
		loadingWindow.setPosition((getWidth()-loadingWindow.getWidth())/2f, (getHeight()-loadingWindow.getHeight())/2f);
		addActor(loadingWindow);
		loadingWindow.playOpenSound();
		loadingWindow.setMovable(false);
		loadingWindow.setModal(true);
		
		musicChoices = new Table();
		musicChoices.defaults().pad(10f);
		musicChoiceScroller = new ScrollPane(musicChoices);
		musicChoiceScroller.setScrollingDisabled(false, true);
		musicChoiceScroller.setSize(getWidth(), getHeight()-(getHeight()-back.getY()));
		
		addActor(musicChoiceScroller);
		
		loadingWindow.toFront();
		
		back.toFront();
	}
	
	@Override
	public void act(float delta) {
		back.toFront();
		super.act(delta);
	}
	
	public void beginMusicSearch() {
		new Thread(new Runnable() {

			@Override
			public void run() {
				Logger.getLogger("org.jaudiotagger").setLevel(Level.SEVERE);
				
				FileHandle[] musicFiles = new FileHandle(core.getPrefs().getString("music dir")).list(new FilenameFilter() {
					@Override
					public boolean accept(File dir, String name) {
						if (name.endsWith("mp3") || name.endsWith("wav")) {
							return true;
						}
						return false;
					}
				});
				
				if (musicFiles.length > 0) {
					for (int music = 0; music < musicFiles.length && !cancel; music++) {
						final MusicSelectable selectable = new MusicSelectable(musicFiles[music], musicFileAnnotation, core.getDefaultSkin(), core.getAssetManager().get("defaultCover.png", Texture.class));
						Gdx.app.postRunnable(new Runnable() {
							@Override
							public void run() {
								Gdx.app.debug("Music Search Thread", "Finished loading: " + selectable.getName());
								musicChoices.add(selectable).prefSize(panelWidthCalc(getWidth()), musicChoiceScroller.getHeight());
								selectable.addInfoToPanel(panelWidthCalc(getWidth()) - 20f);
								
								selectable.addListener(new ChangeListener() {
									@Override
									public void changed(ChangeEvent event, Actor actor) {
										
										if (!selectable.isMusicInvalid()) {
											selectedMusic = selectable.getMusicFile();
											selectedMusicInfo = selectable.getAudioInfo();
											miniSender.send(MiniEvents.MUSIC_SELECTED);
										} else {
											//Play "no" sound
										}
									}
								});
							}
						});
						
						Gdx.app.debug("Music Search Thread", "Completed: " + music);
						int prog = (int) (100f*music/(musicFiles.length-1f));
						loadingWindow.setProgress(prog);
					}
				} else {
					NoticeWindow notice = new NoticeWindow(core.getDefaultSkin(), "default", "No song's found in:\n\"" + core.getPrefs().getString("music dir") + "\"\nTo change the search directory, go to game options.", core.getPrefs().getFloat("fx vol"), core.getAssetManager());
					notice.setSize(0.6f*getWidth(), 0.6f*getHeight());
					notice.setPosition((getWidth()-notice.getWidth())/2f, (getHeight()-notice.getHeight())/2f);
					notice.setModal(true);
					notice.setMovable(false);
					addActor(notice);
					notice.playOpenSound();
				}
				loadingWindow.remove();
			}
		}).start();
	}
	
	public float panelWidthCalc(float origWidth) {
		return (float) (Math.sqrt(getWidth()*35f)+80f);
	}
	
	public FileHandle getSelectedMusic() {
		return selectedMusic;
	}
	
	public AudioInfo getSelectedMusicInfo() {
		return selectedMusicInfo;
	}
	
	@Override
	public synchronized void addActor(Actor actor) {
		super.addActor(actor);
	}
}
